//`timescale 1ns / 1ps
module vga1 (clk,clr,sync,blank,red,green,blue,hsync,vsync,vgaclk,rdb,pixloc,m9krdb,m9kwrb,pixlin,memclk);

output reg sync,blank;
output reg [9:0] red,green,blue;
output wire hsync,vsync;
output wire vgaclk; 
output reg rdb;
input wire clk,clr,memclk;  //
input wire [23:0] pixlin;
output reg [18:0] pixloc;  //location in screen buffer
reg [18:0] oldpixloc;
output reg m9krdb,m9kwrb;
reg getline,linevalid;
reg pixvalid;
reg [31:0] lcnt;
reg [11:0] pcnt;
reg [31:0] fcnt; //frame counter
reg[15:0] pixels3, pixels2;
reg[31:0] vga[12];  // internal regs
reg state1,state2,state3,state4,state5,state6,state7;
reg [2:0]clkdiv;
reg oldhc;
reg [9:0]wbi,rbi;
// video structure constants 40MHz 800x600

parameter hpixels = 1056;// horizontal pixels per line800
parameter vlines = 628; // vertical lines per frame 521
parameter hpulse = 128; 	// hsync pulse length
parameter vpulse =3; 	// vsync pulse length
parameter hbp = 216; 	// end of horizontal back porch144//216 for 800x600
parameter hfp = 1016; 	// beginning of horizontal front porch784
parameter vbp = 23; 		// end of vertical back porch31
parameter vfp = 627; 	// beginning of vertical front porch 511
parameter ppl = 640;    //image pixels per line in frame buffer
parameter lpf = 480;    //image lines in frame buffer
parameter originy = 60;
parameter originx = 0;
// active horizontal video is therefore:hfp-hbp = 1016-216=800

// active vertical video is therefore: 627-23-4 = 600
// registers for storing the horizontal & vertical counters


/*
parameter hpixels = 800;// horizontal pixels per line800
parameter vlines = 521; // vertical lines per frame 521
parameter hpulse = 96; 	// hsync pulse length
parameter vpulse =2; 	// vsync pulse length
parameter hbp = 144; 	// end of horizontal back porch144//216 for 800x600
parameter hfp = 784; 	// beginning of horizontal front porch784
parameter vbp = 31; 		// end of vertical back porch31
parameter vfp = 511; 	// beginning of vertical front porch 511
parameter ppl = 640;    //image pixels per line in frame buffer
parameter lpf = 480;    //image lines in frame buffer
parameter originy = 0;

*/
// active horizontal video is therefore: 784 - 144 = 640
//parameter originx = 0;

// active vertical video is therefore: 511 - 31 = 480
// registers for storing the horizontal & vertical counters
reg [12:0] hc;
reg [12:0] vc,oldvc;
reg[7:0]  linebuff[0:639];

initial
begin
  linevalid = 0;
  fcnt=0;
end

always @(posedge clk)
begin
   oldvc<=vc;
	
	// reset condition
	
	if (clr == 1)
	begin
		hc <= 0;
		vc <= 0;
	
		
	
		
	end
	else
	begin
      if (vc ==1) begin fcnt<=fcnt+1; end
		if (hc < hpixels-1) begin hc <= hc + 1; end//
			 
		else
		// When we hit the end of the line, reset the horizontal
		// counter and increment the vertical counter.
		// If vertical counter is at the end of the frame, then
		// reset that one too.
		
		begin
			hc <= 0;
			if (vc < vlines - 1)
			 begin
				vc <= vc + 1;
			 end
			else
			  begin
				vc <= 0;
				
				
		    
			  end
		end
		
	end
end


// ----------------
// "assign" statements are a quick way to
// give values to variables of type: wire
assign hsync = (hc < hpulse) ? 0:1;
assign vsync = (vc < vpulse) ? 0:1;

assign vgaclk = clk;
always@(posedge clk)
begin
  if ((vc<vbp) || (vc>vfp)||(hc<hbp)|| (hc>hfp))
  begin
    blank<=0;
  end
  else 
  begin
    blank<=1;
  end 
end 
always @(posedge clk)
begin
	// first check if we're within vertical active video range
	if (vc < vbp) begin pixloc<=0;end
	if ( (vc> vbp+originy) && ( vc<=vbp+originy+lpf  )   )
	   begin
		 m9krdb<=1; 
		 if (hc>(hbp+originx-1) && hc <(hbp+originx+ppl))//
			begin 
			//	red <= pixlin[7:0];//
			//	green <= pixlin[7:0];//
			//	blue <= pixlin[7:0];	//
			   red<=vc;
				green<=vc;
				blue<=hc;
				pixloc<=pixloc+1;
		  end
		 else if (hc>(hbp+originx+ppl)&& (hc < (1+hfp)))
		 begin
		     
		 		red <=   10'b0111111111;
				green <= 10'b0111111111;
				blue <=  10'b1111111111;;
			
		 end
	
		 else if ((hc<=hbp) || (hc >=(hfp)))
			begin
	         red <=   10'b0001111111;
				green <= 10'b1111111111;
				blue <=  10'b0111111111;;				
			end
	   end
	// we're outside active vertical image s
	else
	begin
	
	      	red <=   8'b01111111;
				green <= 8'b01111111;
				blue <=  8'b11111111;;		
		
	end
	
end

endmodule
